import asyncio
import time
from abc import ABC, abstractmethod
from collections.abc import Awaitable, Callable
from typing import Any, TypeVar

import structlog
from tenacity import (
    AsyncRetrying,
    before_sleep_log,
    retry_if_exception_type,
    stop_after_attempt,
    wait_exponential,
)

logger = structlog.get_logger()

T = TypeVar("T")


class CircuitBreakerError(Exception):
    pass


class CircuitBreaker:

    def __init__(
        self,
        failure_threshold: int = 5,
        recovery_timeout: int = 60,
        name: str = "circuit_breaker",
    ) -> None:
        self.failure_threshold = failure_threshold
        self.recovery_timeout = recovery_timeout
        self.name = name

        self._failure_count = 0
        self._last_failure_time: float | None = None
        self._state: str = "closed"

    async def call(self, func: Callable[[], Awaitable[T]]) -> T:
        if self._state == "open":
            if self._should_attempt_reset():
                self._state = "half_open"
                logger.info("Circuit breaker перешёл в half_open", name=self.name)
            else:
                raise CircuitBreakerError(f"Circuit breaker {self.name} открыт")

        try:
            result = await func()
            self._on_success()
            return result
        except Exception as e:
            self._on_failure()
            raise e

    def _should_attempt_reset(self) -> bool:
        if self._last_failure_time is None:
            return True
        return time.time() - self._last_failure_time >= self.recovery_timeout

    def _on_success(self) -> None:
        if self._state == "half_open":
            self._state = "closed"
            logger.info("Circuit breaker восстановлен", name=self.name)
        self._failure_count = 0

    def _on_failure(self) -> None:
        self._failure_count += 1
        self._last_failure_time = time.time()

        if self._failure_count >= self.failure_threshold:
            self._state = "open"
            logger.warning(
                "Circuit breaker открыт из-за превышения порога ошибок",
                name=self.name,
                failure_count=self._failure_count,
                threshold=self.failure_threshold,
            )


class RateLimiter:

    def __init__(self, max_concurrent: int, name: str = "rate_limiter") -> None:
        self.semaphore = asyncio.Semaphore(max_concurrent)
        self.name = name
        self.max_concurrent = max_concurrent

    async def __aenter__(self) -> None:
        await self.semaphore.acquire()

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        self.semaphore.release()


async def with_retry(
    func: Callable[[], Awaitable[T]],
    max_attempts: int = 3,
    min_wait: float = 1.0,
    max_wait: float = 10.0,
    retry_exceptions: tuple[type[Exception], ...] = (Exception,),
    name: str = "retry_operation",
) -> T:
    async for attempt in AsyncRetrying(
        stop=stop_after_attempt(max_attempts),
        wait=wait_exponential(multiplier=1, min=min_wait, max=max_wait),
        retry=retry_if_exception_type(retry_exceptions),
        before_sleep=before_sleep_log(logger, "warning"),
        reraise=True,
    ):
        with attempt:
            logger.debug(
                "Попытка выполнения операции",
                operation=name,
                attempt_number=attempt.retry_state.attempt_number,
            )
            return await func()


class BaseAdapter(ABC):

    def __init__(self, name: str) -> None:
        self.name = name
        self.logger = structlog.get_logger().bind(adapter=name)

    @abstractmethod
    async def health_check(self) -> bool:
        pass
