import asyncio
import json
import sys

import click
import structlog

from .dependency_injection import get_container
from .models import AppConfig


def setup_logging(log_level: str, log_format: str) -> None:
    processors = [
        structlog.stdlib.filter_by_level,
        structlog.stdlib.add_logger_name,
        structlog.stdlib.add_log_level,
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.TimeStamper(fmt="iso"),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
    ]

    if log_format == "json":
        processors.append(structlog.processors.JSONRenderer())
    else:
        processors.append(structlog.dev.ConsoleRenderer())

    structlog.configure(
        processors=processors,
        wrapper_class=structlog.stdlib.BoundLogger,
        logger_factory=structlog.stdlib.LoggerFactory(),
        cache_logger_on_first_use=True,
    )

    import logging

    logging.basicConfig(level=getattr(logging, log_level.upper()))


@click.group()
@click.option(
    "--config-file",
    type=click.Path(exists=True),
    help="Путь к файлу конфигурации .env",
)
@click.option("--log-level", default="INFO", help="Уровень логирования")
@click.option(
    "--log-format", default="text", type=click.Choice(["json", "text"]), help="Формат логов"
)
@click.pass_context
def main(ctx: click.Context, config_file: str | None, log_level: str, log_format: str) -> None:
    setup_logging(log_level, log_format)

    if config_file:
        config = AppConfig(_env_file=config_file)
    else:
        config = AppConfig()

    ctx.ensure_object(dict)
    ctx.obj["config"] = config


@main.command()
@click.argument("input_file", type=click.Path(exists=True))
@click.option(
    "--force",
    is_flag=True,
    help="Принудительно обработать даже уже обработанные статьи",
)
@click.option(
    "--max-articles",
    type=int,
    help="Максимальное количество статей для обработки",
)
@click.option(
    "--max-workers",
    type=int,
    help="Максимальное количество worker корутин",
)
@click.pass_context
def process(
    ctx: click.Context,
    input_file: str,
    force: bool,
    max_articles: int | None,
    max_workers: int | None,
) -> None:
    config: AppConfig = ctx.obj["config"]

    async def _run() -> None:
        container = get_container(config)

        try:
            await container.initialize()

            runner = container.create_runner(max_workers)

            click.echo(f"Запуск обработки статей из файла: {input_file}")
            click.echo(f"Принудительная обработка: {force}")
            click.echo(f"Максимум статей: {max_articles or 'без ограничений'}")
            click.echo(f"Workers: {runner.max_workers}")
            click.echo()

            stats = await runner.run_from_file(
                input_file=input_file,
                force_reprocess=force,
                max_articles=max_articles,
            )

            click.echo("\n" + "=" * 50)
            click.echo("РЕЗУЛЬТАТЫ ОБРАБОТКИ")
            click.echo("=" * 50)
            click.echo(f"Всего обработано: {stats.total_processed}")
            click.echo(f"Успешно: {stats.successful}")
            click.echo(f"Ошибок: {stats.failed}")
            click.echo(f"Пропущено: {stats.skipped}")
            click.echo(f"Процент успеха: {stats.success_rate:.1f}%")

            if stats.successful > 0:
                click.echo(f"Среднее время обработки: {stats.average_processing_time:.2f}с")

        except Exception as e:
            click.echo(f"Ошибка: {e}", err=True)
            sys.exit(1)
        finally:
            await container.cleanup()

    asyncio.run(_run())


@main.command()
@click.pass_context
def health(ctx: click.Context) -> None:
    config: AppConfig = ctx.obj["config"]

    async def _check() -> None:
        container = get_container(config)

        try:
            await container.initialize()

            click.echo("Проверка работоспособности системы...")
            checks = await container.health_check()

            click.echo("\nРезультаты проверки:")
            all_ok = True

            for component, status in checks.items():
                status_str = "✓ OK" if status else "✗ FAILED"
                click.echo(f"  {component}: {status_str}")
                if not status:
                    all_ok = False

            if all_ok:
                click.echo("\nВсе компоненты работают нормально")
            else:
                click.echo("\nОбнаружены проблемы с компонентами")
                sys.exit(1)

        except Exception as e:
            click.echo(f"Ошибка при проверке: {e}", err=True)
            sys.exit(1)
        finally:
            await container.cleanup()

    asyncio.run(_check())


@main.command()
@click.argument("input_file", type=click.Path(exists=True))
@click.pass_context
def stats(ctx: click.Context, input_file: str) -> None:
    from .sources import FileSource

    async def _stats() -> None:
        try:
            source = FileSource(input_file)
            total_urls = await source.count_urls()

            click.echo(f"Файл: {input_file}")
            click.echo(f"Валидных URL: {total_urls}")

        except Exception as e:
            click.echo(f"Ошибка: {e}", err=True)
            sys.exit(1)

    asyncio.run(_stats())


@main.command()
@click.option(
    "--limit",
    type=int,
    default=10,
    help="Количество статей для вывода",
)
@click.option(
    "--status",
    type=click.Choice(["pending", "processing", "completed", "failed"]),
    help="Фильтр по статусу",
)
@click.option(
    "--format",
    "output_format",
    type=click.Choice(["table", "json"]),
    default="table",
    help="Формат вывода",
)
@click.pass_context
def list_articles(
    ctx: click.Context,
    limit: int,
    status: str | None,
    output_format: str,
) -> None:
    config: AppConfig = ctx.obj["config"]

    async def _list() -> None:
        container = get_container(config)

        try:
            await container.initialize()
            repository = container.get_repository()

            if status:
                from .models.article_dto import ArticleStatus

                status_enum = ArticleStatus(status)
                articles = await repository.get_articles_by_status(status_enum, limit)
            else:
                articles = await repository.get_all_articles(limit)

            if output_format == "json":
                data = [
                    {
                        "id": article.id,
                        "url": article.url,
                        "title": article.title,
                        "status": article.status.value,
                        "created_at": article.created_at.isoformat(),
                        "simplified_text": article.simplified_text,
                    }
                    for article in articles
                ]
                click.echo(json.dumps(data, ensure_ascii=False, indent=2))
            else:
                if not articles:
                    click.echo("Статьи не найдены")
                    return

                click.echo(f"{'ID':<5} {'Статус':<12} {'Название':<50}")
                click.echo("-" * 72)

                for article in articles:
                    title = article.title[:47] + "..." if len(article.title) > 50 else article.title

                    click.echo(f"{article.id:<5} {article.status.value:<12} {title:<50}")

        except Exception as e:
            click.echo(f"Ошибка: {e}", err=True)
            sys.exit(1)
        finally:
            await container.cleanup()

    asyncio.run(_list())


if __name__ == "__main__":
    main()
