from functools import lru_cache

import structlog

from .adapters import LLMProviderAdapter, RuWikiAdapter
from .models import AppConfig
from .runner import AsyncRunner
from .services import ArticleRepository, AsyncWriteQueue, DatabaseService, SimplifyService

logger = structlog.get_logger()


class DependencyContainer:
    def __init__(self, config: AppConfig) -> None:
        self.config = config
        self._database_service: DatabaseService | None = None
        self._repository: ArticleRepository | None = None
        self._write_queue: AsyncWriteQueue | None = None
        self._ruwiki_adapter: RuWikiAdapter | None = None
        self._llm_adapter: LLMProviderAdapter | None = None
        self._simplify_service: SimplifyService | None = None
        self._initialized = False

    async def initialize(self) -> None:
        if self._initialized:
            return

        logger.info("Инициализация системы...")

        db_service = self.get_database_service()
        await db_service.initialize_database()

        write_queue = self.get_write_queue()
        await write_queue.start()

        self._initialized = True
        logger.info("Система инициализирована")

    async def cleanup(self) -> None:
        if not self._initialized:
            return

        logger.info("Очистка ресурсов...")

        if self._write_queue:
            await self._write_queue.stop()

        self._initialized = False
        logger.info("Ресурсы очищены")

    @lru_cache(maxsize=1)
    def get_database_service(self) -> DatabaseService:
        if self._database_service is None:
            self._database_service = DatabaseService(self.config)
        return self._database_service

    @lru_cache(maxsize=1)
    def get_repository(self) -> ArticleRepository:
        if self._repository is None:
            db_service = self.get_database_service()
            self._repository = ArticleRepository(db_service)
        return self._repository

    @lru_cache(maxsize=1)
    def get_write_queue(self) -> AsyncWriteQueue:
        if self._write_queue is None:
            repository = self.get_repository()
            self._write_queue = AsyncWriteQueue(repository, max_batch_size=10)
        return self._write_queue

    @lru_cache(maxsize=1)
    def get_ruwiki_adapter(self) -> RuWikiAdapter:
        if self._ruwiki_adapter is None:
            self._ruwiki_adapter = RuWikiAdapter(self.config)
        return self._ruwiki_adapter

    @lru_cache(maxsize=1)
    def get_llm_adapter(self) -> LLMProviderAdapter:
        if self._llm_adapter is None:
            self._llm_adapter = LLMProviderAdapter(self.config)
        return self._llm_adapter

    @lru_cache(maxsize=1)
    def get_simplify_service(self) -> SimplifyService:
        if self._simplify_service is None:
            self._simplify_service = SimplifyService(
                config=self.config,
                ruwiki_adapter=self.get_ruwiki_adapter(),
                llm_adapter=self.get_llm_adapter(),
                repository=self.get_repository(),
                write_queue=self.get_write_queue(),
            )
        return self._simplify_service

    def create_runner(self, max_workers: int | None = None) -> AsyncRunner:
        if max_workers is None:
            max_workers = min(
                self.config.max_concurrent_llm,
                self.config.max_concurrent_wiki,
                10,
            )

        return AsyncRunner(
            config=self.config,
            simplify_service=self.get_simplify_service(),
            max_workers=max_workers,
        )

    async def health_check(self) -> dict[str, bool]:
        checks = {}

        try:
            db_service = self.get_database_service()
            checks["database"] = await db_service.health_check()
        except Exception:
            checks["database"] = False

        try:
            write_queue = self.get_write_queue()
            checks["write_queue"] = (
                write_queue._worker_task is not None and not write_queue._worker_task.done()
            )
        except Exception:
            checks["write_queue"] = False

        try:
            ruwiki = self.get_ruwiki_adapter()
            checks["ruwiki"] = await ruwiki.health_check()
        except Exception:
            checks["ruwiki"] = False

        try:
            llm = self.get_llm_adapter()
            checks["llm"] = await llm.health_check()
        except Exception:
            checks["llm"] = False

        return checks


def get_container(config: AppConfig | None = None) -> DependencyContainer:
    if config is None:
        config = AppConfig()

    return DependencyContainer(config)
