from dataclasses import dataclass


@dataclass(frozen=True)
class SimplifyCommand:
    url: str
    force_reprocess: bool = False

    def __str__(self) -> str:
        return f"SimplifyCommand(url='{self.url}', force={self.force_reprocess})"


@dataclass
class ProcessingResult:

    url: str
    success: bool
    title: str | None = None
    raw_text: str | None = None
    simplified_text: str | None = None
    token_count_raw: int | None = None
    token_count_simplified: int | None = None
    processing_time_seconds: float | None = None
    error_message: str | None = None

    @classmethod
    def success_result(
        cls,
        url: str,
        title: str,
        raw_text: str,
        simplified_text: str,
        token_count_raw: int,
        token_count_simplified: int,
        processing_time_seconds: float,
    ) -> "ProcessingResult":
        return cls(
            url=url,
            success=True,
            title=title,
            raw_text=raw_text,
            simplified_text=simplified_text,
            token_count_raw=token_count_raw,
            token_count_simplified=token_count_simplified,
            processing_time_seconds=processing_time_seconds,
        )

    @classmethod
    def failure_result(cls, url: str, error_message: str) -> "ProcessingResult":
        return cls(
            url=url,
            success=False,
            error_message=error_message,
        )


@dataclass
class ProcessingStats:

    total_processed: int = 0
    successful: int = 0
    failed: int = 0
    skipped: int = 0
    total_processing_time: float = 0.0

    @property
    def success_rate(self) -> float:
        if self.total_processed == 0:
            return 0.0
        return (self.successful / self.total_processed) * 100.0

    @property
    def average_processing_time(self) -> float:
        if self.successful == 0:
            return 0.0
        return self.total_processing_time / self.successful

    def add_result(self, result: ProcessingResult) -> None:
        self.total_processed += 1

        if result.success:
            self.successful += 1
            if result.processing_time_seconds:
                self.total_processing_time += result.processing_time_seconds
        else:
            self.failed += 1

    def add_skipped(self) -> None:
        self.skipped += 1
