from pathlib import Path
from typing import Literal

from pydantic import Field, field_validator
from pydantic_settings import BaseSettings, SettingsConfigDict


class AppConfig(BaseSettings):

    model_config = SettingsConfigDict(
        env_file=".env",
        env_file_encoding="utf-8",
        case_sensitive=False,
        extra="ignore",
    )

    openai_api_key: str = Field(description="API ключ OpenAI")
    openai_model: str = Field(default="gpt-4o-mini", description="Модель OpenAI для упрощения")
    openai_temperature: float = Field(
        default=0.0, ge=0.0, le=2.0, description="Температура для LLM"
    )
    openai_proxy_url: str | None = Field(description="Proxy URL для OpenAI")

    db_path: str = Field(default="./data/wiki.db", description="Путь к файлу SQLite")

    max_concurrent_llm: int = Field(
        default=5, ge=1, le=50, description="Максимум одновременных LLM запросов"
    )
    openai_rpm: int = Field(default=200, ge=1, description="Лимит запросов в минуту для OpenAI")
    max_concurrent_wiki: int = Field(
        default=10, ge=1, le=100, description="Максимум одновременных wiki запросов"
    )

    log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR"] = Field(default="INFO")
    log_format: Literal["json", "text"] = Field(default="json")

    chunk_size: int = Field(default=2000, ge=500, le=122000, description="Размер чанка для текста")
    chunk_overlap: int = Field(default=200, ge=0, le=1000, description="Перекрытие между чанками")
    max_retries: int = Field(default=3, ge=1, le=10, description="Максимум попыток повтора")
    retry_delay: float = Field(
        default=1.0, ge=0.1, le=60.0, description="Задержка между попытками (сек)"
    )

    circuit_failure_threshold: int = Field(
        default=5, ge=1, le=20, description="Порог отказов для circuit breaker"
    )
    circuit_recovery_timeout: int = Field(
        default=60,
        ge=10,
        le=600,
        description="Время восстановления circuit breaker (сек)",
    )

    prompt_template_path: str = Field(
        default="src/prompt.txt", description="Путь к файлу с prompt-шаблоном"
    )
    input_file_path: str = Field(
        default="input.txt", description="Путь к файлу с URL для обработки"
    )

    @field_validator("db_path")
    @classmethod
    def validate_db_path(cls, v: str) -> str:
        db_path = Path(v)
        db_path.parent.mkdir(parents=True, exist_ok=True)
        return str(db_path)

    @property
    def db_url(self) -> str:
        return f"sqlite+aiosqlite:///{self.db_path}"

    @property
    def sync_db_url(self) -> str:
        return f"sqlite:///{self.db_path}"
