from datetime import datetime, timezone
from typing import Any

import aiosqlite
import structlog

from ..models.article_dto import ArticleDTO, ArticleStatus
from .database import DatabaseService

logger = structlog.get_logger()


class ArticleRepository:

    def __init__(self, db_service: DatabaseService) -> None:
        self.db_service = db_service
        self.logger = structlog.get_logger().bind(repository="article")

    async def create_article(self, url: str, title: str, raw_text: str) -> ArticleDTO:
        existing = await self.get_by_url(url)
        if existing:
            raise ValueError(f"Статья с URL {url} уже существует")

        article = ArticleDTO(
            url=url,
            title=title,
            raw_text=raw_text,
            status=ArticleStatus.PENDING,
            created_at=datetime.now(timezone.utc),
        )

        async with await self.db_service.get_connection() as conn:
            conn.row_factory = aiosqlite.Row
            cursor = await conn.execute(
                """
                INSERT INTO articles (url, title, raw_text, status, created_at)
                VALUES (?, ?, ?, ?, ?)
                """,
                (
                    article.url,
                    article.title,
                    article.raw_text,
                    article.status.value,
                    article.created_at,
                ),
            )
            await conn.commit()

            article.id = cursor.lastrowid

        self.logger.info("Статья создана", article_id=article.id, url=article.url)
        return article

    async def get_by_id(self, article_id: int) -> ArticleDTO | None:
        async with await self.db_service.get_connection() as conn:
            conn.row_factory = aiosqlite.Row
            cursor = await conn.execute(
                "SELECT * FROM articles WHERE id = ?",
                (article_id,),
            )
            row = await cursor.fetchone()

        if not row:
            return None

        return self._row_to_article_dto(row)

    async def get_by_url(self, url: str) -> ArticleDTO | None:
        async with await self.db_service.get_connection() as conn:
            conn.row_factory = aiosqlite.Row
            cursor = await conn.execute(
                "SELECT * FROM articles WHERE url = ?",
                (url,),
            )
            row = await cursor.fetchone()

        if not row:
            return None

        return self._row_to_article_dto(row)

    async def update_article(self, article: ArticleDTO) -> ArticleDTO:
        if not article.id:
            raise ValueError("ID статьи не может быть None для обновления")

        article.updated_at = datetime.now(timezone.utc)

        async with await self.db_service.get_connection() as conn:
            conn.row_factory = aiosqlite.Row
            cursor = await conn.execute(
                """
                UPDATE articles SET
                    title = ?,
                    raw_text = ?,
                    simplified_text = ?,
                    status = ?,
                    updated_at = ?
                WHERE id = ?
                """,
                (
                    article.title,
                    article.raw_text,
                    article.simplified_text,
                    article.status.value,
                    article.updated_at,
                    article.id,
                ),
            )
            await conn.commit()

            if cursor.rowcount == 0:
                raise ValueError(f"Статья с ID {article.id} не найдена")

        self.logger.info("Статья обновлена", article_id=article.id, status=article.status)
        return article

    async def get_articles_by_status(
        self, status: ArticleStatus, limit: int | None = None
    ) -> list[ArticleDTO]:
        query = "SELECT * FROM articles WHERE status = ?"
        params: tuple[Any, ...] = (status.value,)

        if limit:
            query += " LIMIT ?"
            params = params + (limit,)

        async with await self.db_service.get_connection() as conn:
            conn.row_factory = aiosqlite.Row
            cursor = await conn.execute(query, params)
            rows = await cursor.fetchall()

        return [self._row_to_article_dto(row) for row in rows]

    async def get_pending_articles(self, limit: int | None = None) -> list[ArticleDTO]:
        return await self.get_articles_by_status(ArticleStatus.PENDING, limit)

    async def count_by_status(self, status: ArticleStatus) -> int:
        async with await self.db_service.get_connection() as conn:
            conn.row_factory = aiosqlite.Row
            cursor = await conn.execute(
                "SELECT COUNT(*) FROM articles WHERE status = ?",
                (status.value,),
            )
            result = await cursor.fetchone()

        return result[0] if result else 0

    async def get_all_articles(self, limit: int | None = None, offset: int = 0) -> list[ArticleDTO]:
        query = "SELECT * FROM articles ORDER BY created_at DESC"
        params: tuple[Any, ...] = ()

        if limit:
            query += " LIMIT ? OFFSET ?"
            params = (limit, offset)

        async with await self.db_service.get_connection() as conn:
            conn.row_factory = aiosqlite.Row
            cursor = await conn.execute(query, params)
            rows = await cursor.fetchall()

        return [self._row_to_article_dto(row) for row in rows]

    async def delete_article(self, article_id: int) -> bool:
        async with await self.db_service.get_connection() as conn:
            conn.row_factory = aiosqlite.Row
            cursor = await conn.execute(
                "DELETE FROM articles WHERE id = ?",
                (article_id,),
            )
            await conn.commit()

        deleted = cursor.rowcount > 0
        if deleted:
            self.logger.info("Статья удалена", article_id=article_id)

        return deleted

    def _row_to_article_dto(self, row: aiosqlite.Row) -> ArticleDTO:
        return ArticleDTO(
            id=row["id"],
            url=row["url"],
            title=row["title"],
            raw_text=row["raw_text"],
            simplified_text=row["simplified_text"],
            status=ArticleStatus(row["status"]),
            created_at=datetime.fromisoformat(row["created_at"]) if row["created_at"] else None,
            updated_at=datetime.fromisoformat(row["updated_at"]) if row["updated_at"] else None,
        )
