from __future__ import annotations

import asyncio
import time
from pathlib import Path

import structlog

from src.adapters.llm import LLMProviderAdapter, LLMTokenLimitError
from src.adapters.ruwiki import RuWikiAdapter
from src.models import AppConfig, ProcessingResult, SimplifyCommand
from src.models.constants import LLM_MAX_INPUT_TOKENS, MAX_TOKEN_LIMIT_WITH_BUFFER
from src.services.repository import ArticleRepository
from src.services.text_splitter import RecursiveCharacterTextSplitter
from src.services.write_queue import AsyncWriteQueue


class SimplifyService:
    def __init__(
        self,
        config: AppConfig,
        ruwiki_adapter: RuWikiAdapter,
        llm_adapter: LLMProviderAdapter,
        repository: ArticleRepository,
        write_queue: AsyncWriteQueue,
    ) -> None:
        self.config = config
        self.ruwiki_adapter = ruwiki_adapter
        self.llm_adapter = llm_adapter
        self.repository = repository
        self.write_queue = write_queue

        self.text_splitter = RecursiveCharacterTextSplitter(
            chunk_size=config.chunk_size,
            chunk_overlap=config.chunk_overlap,
            length_function=self.llm_adapter.count_tokens,
        )

        self._prompt_template: str | None = None
        self.logger = structlog.get_logger().bind(service="simplify")

    async def get_prompt_template(self) -> str:
        if self._prompt_template is None:
            prompt_path = Path(self.config.prompt_template_path)
            if not prompt_path.exists():
                msg = f"Prompt template не найден: {prompt_path}"
                raise FileNotFoundError(msg)

            self._prompt_template = prompt_path.read_text(encoding="utf-8")

        return self._prompt_template

    async def process_command(self, command: SimplifyCommand) -> ProcessingResult:
        start_time = time.time()
        self.logger.info("Начало обработки статьи", url=command.url)

        try:
            return await self._process_command_impl(command, start_time)
        except Exception as e:
            return await self._handle_processing_error(command, e, start_time)

    async def _process_command_impl(
        self, command: SimplifyCommand, start_time: float
    ) -> ProcessingResult:
        if not command.force_reprocess:
            existing_result = await self._check_existing_article(command.url)
            if existing_result:
                return existing_result

        page_info = await self.ruwiki_adapter.fetch_page_cleaned(command.url)
        article = await self._create_or_update_article(command, page_info)

        from src.models.article_dto import ArticleStatus

        article.status = ArticleStatus.PENDING
        await self.repository.update_article(article)

        simplified_text, input_tokens, output_tokens = await self._simplify_article_text(
            title=page_info.title,
            raw_text=page_info.content,
        )

        self.logger.info(
            "Упрощение завершено",
            url=command.url,
            simplified_length=len(simplified_text),
            input_tokens=input_tokens,
            output_tokens=output_tokens,
        )

        if not simplified_text.strip():
            self.logger.error("Получен пустой simplified_text!", url=command.url)
            raise ValueError("Упрощение привело к пустому результату")

        processing_time = time.time() - start_time
        result = ProcessingResult.success_result(
            url=command.url,
            title=page_info.title,
            raw_text=page_info.content,
            simplified_text=simplified_text,
            token_count_raw=input_tokens,
            token_count_simplified=output_tokens,
            processing_time_seconds=processing_time,
        )

        self.logger.info("Отправляем результат в write_queue...", url=command.url)
        await self.write_queue.update_from_result(result)
        self.logger.info("Результат успешно записан в write_queue", url=command.url)

        self.logger.info(
            "Статья успешно обработана",
            url=command.url,
            title=page_info.title,
            processing_time=processing_time,
            input_tokens=input_tokens,
            output_tokens=output_tokens,
        )

        return result

    async def _check_existing_article(self, url: str) -> ProcessingResult | None:
        existing_article = await self.repository.get_by_url(url)
        if existing_article and existing_article.simplified_text:
            self.logger.info("Статья уже обработана, пропускаем", url=url)
            return ProcessingResult.success_result(
                url=url,
                title=existing_article.title,
                raw_text=existing_article.raw_text,
                simplified_text=existing_article.simplified_text,
                token_count_raw=0,
                token_count_simplified=0,
                processing_time_seconds=0,
            )
        return None

    async def _create_or_update_article(self, command, page_info):
        try:
            return await self.repository.create_article(
                url=command.url,
                title=page_info.title,
                raw_text=page_info.content,
            )
        except ValueError:
            article = await self.repository.get_by_url(command.url)
            if not article:
                msg = f"Не удалось найти статью после создания: {command.url}"
                raise ValueError(msg) from None

            if command.force_reprocess:
                from src.models.article_dto import ArticleStatus

                article.title = page_info.title
                article.raw_text = page_info.content
                article.status = ArticleStatus.PENDING
                await self.repository.update_article(article)

            return article

    async def _handle_processing_error(
        self, command: SimplifyCommand, error: Exception, start_time: float
    ) -> ProcessingResult:
        processing_time = time.time() - start_time
        error_message = f"{type(error).__name__}: {error!s}"

        self.logger.exception(
            "Ошибка при обработке статьи",
            url=command.url,
            processing_time=processing_time,
        )

        error_result = ProcessingResult.failure_result(command.url, error_message)
        try:
            await self.write_queue.update_from_result(error_result)
        except Exception:
            self.logger.exception("Ошибка записи результата с ошибкой")

        return error_result

    async def _simplify_article_text(self, title: str, raw_text: str) -> tuple[str, int, int]:
        prompt_template = await self.get_prompt_template()
        text_tokens = self.llm_adapter.count_tokens(raw_text)
        max_input_size = LLM_MAX_INPUT_TOKENS - len(prompt_template) - 1000
        if text_tokens <= max_input_size:
            return await self.llm_adapter.simplify_text(title, raw_text, prompt_template)

        return await self._process_long_text(title, raw_text, prompt_template)

    async def _process_long_text(
        self, title: str, raw_text: str, prompt_template: str
    ) -> tuple[str, int, int]:
        self.logger.info(
            "Разбиение длинного текста на части",
            title=title,
            total_tokens=self.llm_adapter.count_tokens(raw_text),
            chunk_size=self.config.chunk_size,
        )

        chunks = self.text_splitter.split_text(raw_text)
        simplified_chunks = []
        total_input_tokens = 0
        total_output_tokens = 0

        for i, chunk in enumerate(chunks):
            self.logger.debug(
                "Обработка части текста",
                title=title,
                chunk_index=i + 1,
                total_chunks=len(chunks),
                chunk_tokens=self.llm_adapter.count_tokens(chunk),
            )

            try:
                simplified_chunk, input_tokens, output_tokens = (
                    await self.llm_adapter.simplify_text(
                        title=f"{title} (часть {i+1}/{len(chunks)})",
                        wiki_text=chunk,
                        prompt_template=prompt_template,
                    )
                )

                simplified_chunks.append(simplified_chunk)
                total_input_tokens += input_tokens
                total_output_tokens += output_tokens

            except Exception as e:
                self.logger.warning(
                    "Ошибка при обработке части текста",
                    title=title,
                    chunk_index=i + 1,
                    error=str(e),
                )

        if not simplified_chunks:
            msg = "Не удалось обработать ни одной части текста"
            raise LLMTokenLimitError(msg)

        combined_text = self._combine_simplified_chunks(simplified_chunks)
        return self._ensure_token_limit(combined_text, total_input_tokens, total_output_tokens)

    def _ensure_token_limit(
        self, combined_text: str, total_input_tokens: int, total_output_tokens: int
    ) -> tuple[str, int, int]:
        final_tokens = self.llm_adapter.count_tokens(combined_text)
        if final_tokens > MAX_TOKEN_LIMIT_WITH_BUFFER:
            self.logger.warning(
                "Объединённый текст превышает лимит, обрезаем",
                final_tokens=final_tokens,
            )
            combined_text = self._truncate_to_token_limit(
                combined_text, MAX_TOKEN_LIMIT_WITH_BUFFER
            )
            total_output_tokens = self.llm_adapter.count_tokens(combined_text)

        return combined_text, total_input_tokens, total_output_tokens

    def _combine_simplified_chunks(self, chunks: list[str]) -> str:
        combined = "\n\n".join(chunk.strip() for chunk in chunks if chunk.strip())
        return "\n".join(line for line in combined.split("\n") if line.strip())

    def _truncate_to_token_limit(self, text: str, token_limit: int) -> str:
        current_tokens = self.llm_adapter.count_tokens(text)
        if current_tokens <= token_limit:
            return text

        sentences = text.split(". ")
        truncated = ""

        for sentence in sentences:
            test_text = truncated + sentence + ". "
            if self.llm_adapter.count_tokens(test_text) > token_limit:
                break
            truncated = test_text

        return truncated.strip()

    async def health_check(self) -> dict[str, bool]:
        checks = {}

        checks["ruwiki"] = await self._safe_health_check(self.ruwiki_adapter.health_check)
        checks["llm"] = await self._safe_health_check(self.llm_adapter.health_check)
        checks["prompt_template"] = await self._safe_health_check(self.get_prompt_template)

        return checks

    async def _safe_health_check(self, check_func) -> bool:
        try:
            await check_func()
            return True
        except Exception:
            return False
