import re
from collections.abc import Callable

import structlog

logger = structlog.get_logger()


class RecursiveCharacterTextSplitter:
    def __init__(
        self,
        chunk_size: int = 2000,
        chunk_overlap: int = 200,
        length_function: Callable[[str], int] | None = None,
        separators: list[str] | None = None,
    ) -> None:
        self.chunk_size = chunk_size
        self.chunk_overlap = chunk_overlap
        self.length_function = length_function or len

        self.separators = separators or [
            "\n\n",
            "\n",
            ". ",
            "! ",
            "? ",
            "; ",
            ", ",
            " ",
            "",
        ]

        self.logger = structlog.get_logger().bind(service="text_splitter")

    def split_text(self, text: str) -> list[str]:
        if not text.strip():
            return []

        if self.length_function(text) <= self.chunk_size:
            return [text]

        chunks = self._split_text_recursive(text, self.separators)

        merged_chunks = self._merge_splits(chunks)

        self.logger.debug(
            "Текст разбит на части",
            original_length=self.length_function(text),
            chunks_count=len(merged_chunks),
            avg_chunk_size=(
                sum(self.length_function(chunk) for chunk in merged_chunks) / len(merged_chunks)
                if merged_chunks
                else 0
            ),
        )

        return merged_chunks

    def _split_text_recursive(self, text: str, separators: list[str]) -> list[str]:
        final_chunks = []
        separator = separators[-1]
        new_separators = []

        for i, sep in enumerate(separators):
            if sep == "":
                separator = sep
                break
            if re.search(re.escape(sep), text):
                separator = sep
                new_separators = separators[i + 1 :]
                break

        splits = self._split_by_separator(text, separator)

        good_splits = []
        for split in splits:
            if self.length_function(split) < self.chunk_size:
                good_splits.append(split)
            else:
                if good_splits:
                    merged_text = self._merge_splits(good_splits)
                    final_chunks.extend(merged_text)
                    good_splits = []

                if not new_separators:
                    final_chunks.extend(self._split_by_length(split))
                else:
                    other_info = self._split_text_recursive(split, new_separators)
                    final_chunks.extend(other_info)

        if good_splits:
            merged_text = self._merge_splits(good_splits)
            final_chunks.extend(merged_text)

        return final_chunks

    def _split_by_separator(self, text: str, separator: str) -> list[str]:
        if separator == "":
            return list(text)

        return text.split(separator)

    def _split_by_length(self, text: str) -> list[str]:
        chunks = []
        start = 0

        while start < len(text):
            end = start + self.chunk_size

            if end < len(text):
                for offset in range(min(100, self.chunk_size // 4)):
                    if end - offset > start and text[end - offset] in " \n\t.,;!?":
                        end = end - offset + 1
                        break

            chunk = text[start:end].strip()
            if chunk:
                chunks.append(chunk)

            start = max(start + 1, end - self.chunk_overlap)

        return chunks

    def _merge_splits(self, splits: list[str]) -> list[str]:
        if not splits:
            return []

        merged_chunks = []
        current_chunk = ""

        for split in splits:
            test_chunk = current_chunk
            if current_chunk and not current_chunk.endswith(("\n", " ")):
                if not split.startswith(("\n", " ")):
                    test_chunk += " "
            test_chunk += split

            if self.length_function(test_chunk) <= self.chunk_size:
                current_chunk = test_chunk
            else:
                if current_chunk.strip():
                    merged_chunks.append(current_chunk.strip())

                current_chunk = split

        if current_chunk.strip():
            merged_chunks.append(current_chunk.strip())

        return merged_chunks

    def create_chunks_with_metadata(self, text: str, title: str = "") -> list[dict[str, str]]:
        chunks = self.split_text(text)

        return [
            {
                "text": chunk,
                "title": title,
                "chunk_index": i,
                "total_chunks": len(chunks),
                "chunk_size": self.length_function(chunk),
            }
            for i, chunk in enumerate(chunks)
        ]
